@@ -4,7 +4,7 @@
 
 package Convert::ASN1;
 {
-  $Convert::ASN1::VERSION = '0.23';
+  $Convert::ASN1::VERSION = '0.24';
 }
 
 use 5.004;
@@ -49,7 +49,7 @@ BEGIN {
 
   @opName = qw(
     opUNKNOWN opBOOLEAN opINTEGER opBITSTR opSTRING opNULL opOBJID opREAL
-    opSEQUENCE opSET opUTIME opGTIME opUTF8 opANY opCHOICE opROID opBCD
+    opSEQUENCE opEXPLICIT opSET opUTIME opGTIME opUTF8 opANY opCHOICE opROID opBCD
     opEXTENSIONS
   );
 
@@ -120,6 +120,15 @@ sub configure {
     Carp::croak("Unsupported encoding format '$opt{encoding}'");
   }
 
+  # IMPLICIT as defalt for backwards compatibility, even though it's wrong.
+  $self->{options}{tagdefault} = uc($opt{tagdefault} || 'IMPLICIT');
+
+  unless ($self->{options}{tagdefault} =~ /^(?:EXPLICIT|IMPLICIT)$/) {
+    require Carp;
+    Carp::croak("Default tagging must be EXPLICIT/IMPLICIT. Not $opt{tagdefault}");
+  }
+
+
   for my $type (qw(encode decode)) {
     if (exists $opt{$type}) {
       while(my($what,$value) = each %{$opt{$type}}) {
@@ -150,9 +159,9 @@ sub prepare {
   if( ref($asn) eq 'GLOB' ){
     local $/ = undef;
     my $txt = <$asn>;
-    $tree = Convert::ASN1::parser::parse($txt);
+    $tree = Convert::ASN1::parser::parse($txt,$self->{options}{tagdefault});
   } else {
-    $tree = Convert::ASN1::parser::parse($asn);
+    $tree = Convert::ASN1::parser::parse($asn,$self->{options}{tagdefault});
   }
 
   unless ($tree) {
@@ -254,16 +263,27 @@ sub asn_encode_length {
 
 sub decode {
   my $self  = shift;
+  my $ret;
 
   local $SIG{__DIE__};
-  my $ret = eval { 
+  eval {
     my (%stash, $result);
     my $script = $self->{script};
-    my $stash = (1 == @$script && !$self->{script}[0][cVAR]) ? \$result : ($result=\%stash);
+    my $stash = \$result;
+
+    while ($script) {
+      my $child = $script->[0] or last;
+      if (@$script > 1 or defined $child->[cVAR]) {
+        $result = $stash = \%stash;
+        last;
+      }
+      last if $child->[cTYPE] == opCHOICE;
+      $script = $child->[cCHILD];
+    }
 
     _decode(
 	$self->{options},
-	$script,
+	$self->{script},
 	$stash,
 	0,
 	length $_[0], 
@@ -271,12 +291,10 @@ sub decode {
 	{},
 	$_[0]);
 
-    $result;
-  };
-  if ($@) {
-    $self->{'error'} = $@;
-    return undef;
-  }
+    $ret = $result;
+    1;
+  } or $self->{'error'} = $@ || 'Unknown error';
+
   $ret;
 }
 
